/**********************************************************************
 * quit.c                                                  October 1999
 * Horms                                             horms@vergenet.net
 *
 * Protocol independent quit
 *
 * Released under the terms of the GNU GPL
 *
 **********************************************************************/

#include "quit.h"
#include "options.h"


/**********************************************************************
 * quit
 * Send a quit request to a server
 * Pre: fd: file descriptor to write to
 * Return 0 on success
 *        -1 on error
 **********************************************************************/

int quit(const int out_fd, const int in_fd, protocol_t *protocol){
  token_t *t;
  queue *q;
  int status;

  extern options_t opt;

  if(write_str(out_fd,
    NULL_FLAG,
    3,
    protocol->one_time_tag,
    " ",
    protocol->quit_string
  )<0){
    if(opt.debug){
      syslog(LOG_WARNING, "quit: write_str");
    }
    return(-1);
  }
 
  /* We need to read the response, even though we don't
   * care about it
   */

  status=-1;

  if((t=create_token())==NULL){
    if(opt.debug){
      syslog(LOG_WARNING, "quit: create_token");
    }
    goto leave;
  }
  assign_token(
    t, 
    (PERDITION_USTRING) protocol->type[PROTOCOL_OK], 
    strlen(protocol->type[PROTOCOL_OK]), 
    -1
  );

  if((protocol->out_response(in_fd,protocol->one_time_tag,t,&q,NULL,NULL))<0){
    if(opt.debug){
      syslog(LOG_WARNING, "quit: out_response");
    }
    goto leave;
  }

  status=0;

  leave:
  unassign_token(t);
  destroy_token(&t);
  destroy_queue(q, DESTROY_TOKEN);
  return(status);
}
